#!/bin/bash
CERTFILE=$1
CHAINFILE=$2
ENVOY_CONFIG=/envoy/config/envoy-config.yaml
ENVOY_CONFIG_OCSP=/envoy/config/envoy-config-ocsp.yaml
export ENVOY_CONFIG
filesPresent=0
OCSP_RESPONSE=/data/certs/good_ocsp_resp.der
OCSP_CHECK_LOGFILE=/data/logs/ocsp.log
OCSP_REQ_FILE=/data/logs/oscp.req

echo ================================================= >>$OCSP_CHECK_LOGFILE
echo $(date "+%Y-%m-%d %H:%M:%S") >>$OCSP_CHECK_LOGFILE

function urlencode() {
  old_lc_collate=$LC_COLLATE
  LC_COLLATE=C
  local length="${#1}"
  for ((i = 0; i < length; i++)); do
    local c="${1:$i:1}"
    case $c in
    [a-zA-Z0-9.~_-]) printf '%s' "$c" ;;
    *) printf '%%%02X' "'$c" ;;
    esac
  done
  LC_COLLATE=$old_lc_collate
}

function gen_der_with_proxy() {
  ocsp_responder_url="$1"
  proxy_url="$2"
  openssl ocsp -noverify -no_nonce -reqout $OCSP_REQ_FILE -issuer "$CHAINFILE" -cert "$CERTFILE" -url "$ocsp_responder_url" 2>/dev/null
  openssl enc -in "$OCSP_REQ_FILE" -out "$OCSP_REQ_FILE.b64" -a
  formatted_req=$(tr -d '\n' <"$OCSP_REQ_FILE.b64")
  url_encoded_req=$(urlencode "$formatted_req")
  status_code=$(curl -s --url "$ocsp_responder_url/$url_encoded_req" --output "$OCSP_RESPONSE.tmp" -w "%{http_code}")
  rm $OCSP_REQ_FILE "$OCSP_REQ_FILE.b64"
  openssl ocsp -respin "$OCSP_RESPONSE.tmp" -issuer "$CHAINFILE" -no_nonce
  exit_code=$?
  if [ "$exit_code" -eq 0 ] && [ "$status_code" = "200" ]; then
    cp "$OCSP_RESPONSE.tmp" $OCSP_RESPONSE
    export ENVOY_CONFIG=$ENVOY_CONFIG_OCSP
    echo "OCSP response captured through http(s) proxy $proxy_url ." >>$OCSP_CHECK_LOGFILE
  else
    echo "Received responder error in OCSP response, therefore, starting envoy in non OCSP Stapling mode" >>$OCSP_CHECK_LOGFILE
  fi
  rm "$OCSP_RESPONSE.tmp"
}

function check() {
  local certOk=0
  local chainOk=0

  if [ -f "$CERTFILE" ]; then
    certOk=1
  fi

  if [ -f "$CHAINFILE" ]; then
    chainOk=1
  fi

  filesPresent=$(($certOk && $chainOk))
}

check

if [[ $filesPresent == 1 ]]; then
  echo "Certificate chain found, performing OCSP stapling" >>$OCSP_CHECK_LOGFILE
  openssl x509 -noout -ocsp_uri -in $CERTFILE >/data/logs/ocsp_endpoint.txt
  ocsp_responder_url=$(</data/logs/ocsp_endpoint.txt)
  echo "OCSP responder URL=$ocsp_responder_url" >>$OCSP_CHECK_LOGFILE
  rm /data/logs/ocsp_endpoint.txt
  if [ ! -z "$ocsp_responder_url" ]; then
    if [ ! -z $HTTP_PROXY ]; then
      gen_der_with_proxy "$ocsp_responder_url" "$HTTP_PROXY"
    elif [ ! -z $HTTPS_PROXY ]; then
      gen_der_with_proxy "$ocsp_responder_url" "$HTTPS_PROXY"
    else
      openssl ocsp -issuer $CHAINFILE -cert $CERTFILE -verify_other $CHAINFILE -CAfile $CHAINFILE -no_nonce -url $ocsp_responder_url -respout $OCSP_RESPONSE
      echo "OCSP response captured" >>$OCSP_CHECK_LOGFILE
      export ENVOY_CONFIG=$ENVOY_CONFIG_OCSP
    fi
  fi
else
  echo "Certificate chain does not exist, hence OCSP stapling is not applicable" >>$OCSP_CHECK_LOGFILE
fi
echo ================================================= >>$OCSP_CHECK_LOGFILE
